# Copyright 2012 GRNET S.A. All rights reserved.
#
# Redistribution and use in source and binary forms, with or
# without modification, are permitted provided that the following
# conditions are met:
#
#   1. Redistributions of source code must retain the above
#      copyright notice, this list of conditions and the following
#      disclaimer.
#
#   2. Redistributions in binary form must reproduce the above
#      copyright notice, this list of conditions and the following
#      disclaimer in the documentation and/or other materials
#      provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY GRNET S.A. ``AS IS'' AND ANY EXPRESS
# OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GRNET S.A OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
# USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
# AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# The views and conclusions contained in the software and
# documentation are those of the authors and should not be
# interpreted as representing official policies, either expressed
# or implied, of GRNET S.A.


from django.contrib.markup.templatetags.markup import textile
from django.db import models
from django import forms
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django.template.loader import render_to_string
from django.utils import simplejson
from django.utils.html import mark_safe

from feincms.admin.editor import ItemEditorForm
from feincms.module.page.models import Page
from feincms.module.medialibrary.models import MediaFile
from feincms.content.medialibrary.models import MediaFileWidget
from feincms.module.medialibrary.fields import MediaFileForeignKey


TWITTER_LOAD_CODE = getattr(settings, 'CLOUDCMS_TWITTER_LOAD_CODE',
"""!function(d,s,id){var js,fjs=d.getElementsByTagName(s)[0],p=/^http:/.test(d.location)?'http':'https';if(!d.getElementById(id)){js=d.createElement(s);js.id=id;js.src=p+"://platform.twitter.com/widgets.js";fjs.parentNode.insertBefore(js,fjs);}}(document,"script","twitter-wjs");""")

class VideoSection(models.Model):
    section_title = models.CharField(max_length=200, blank=True)
    video_title = models.CharField(max_length=200, blank=True)
    video_link = models.CharField(max_length=200, blank=True)
    video_width = models.PositiveIntegerField(default=700)
    video_height = models.PositiveIntegerField(default=550)
    image = MediaFileForeignKey(MediaFile, blank=True, null=True,
    related_name="as_image_for_video_section")
    image_hover = MediaFileForeignKey(MediaFile, blank=True, null=True,
            related_name="as_hover_for_video_section")
    alt_text = models.TextField(null=True, blank=True)
    extra_url_params = models.CharField(max_length=200, blank=True)

    class Meta:
        abstract = True
        verbose_name = _('video section')
        verbose_name_plural = _('video sections')

    def render(self, **kwargs):
        return render_to_string(['content/videosection.html'], {'content': self})


TWITTER_HELP_DESC = mark_safe("""
See <a
href="https://dev.twitter.com/docs/embedded-timelines">embedded timelines
documentation</a>
""")

class TwitterFeed(models.Model):
    title = models.CharField(max_length=200, blank=True)
    widget_id = models.CharField(max_length=200, blank=True,
                                 help_text= mark_safe(_("""
Widget id from
<a href=https://twitter.com/settings/widgets>Twitter widgets</a>
""")))

    limit = models.PositiveIntegerField(default=10,
                                        help_text="Number of tweets to be displayed")
    theme = models.CharField(max_length=200, blank=True, default="light",
                             help_text=TWITTER_HELP_DESC)
    link_color = models.CharField(max_length=200, blank=True, default="#F89A1C",
                             help_text=TWITTER_HELP_DESC)
    chrome = models.CharField(max_length=200, blank=True, default="noscrollbar nofooter noheader noborders",
                             help_text=TWITTER_HELP_DESC)
    screen_name = models.CharField(max_length=200, blank=True,
                             help_text=TWITTER_HELP_DESC)
    custom_timeline_id = models.CharField(max_length=200, blank=True,
                             help_text=TWITTER_HELP_DESC)
    favorites_screen_name = models.CharField(max_length=200, blank=True,
                             help_text=TWITTER_HELP_DESC)
    list_owner_screen_name = models.CharField(max_length=200, blank=True,
                             help_text=TWITTER_HELP_DESC)
    list_slug = models.CharField(max_length=200, blank=True,
                             help_text=TWITTER_HELP_DESC)
    aria_politeness = models.CharField(max_length=200, blank=True,
                             help_text=TWITTER_HELP_DESC)
    border_color = models.CharField(max_length=200, blank=True,
                             help_text=TWITTER_HELP_DESC)
    height = models.CharField(max_length=200, blank=True,
                             help_text=TWITTER_HELP_DESC)
    extra_attributes = models.TextField(blank=True,
            help_text="Extra code to be applied within the html element")

    code = models.TextField(blank=True,
            help_text="Twitter loading code. "
                      "Leave empty for default javascript code to be used.")

    class Meta:
        abstract = True
        verbose_name = _('twitter feed')
        verbose_name_plural = _('twitter feeds')

    def render(self, **kwargs):
        return render_to_string(['content/twitter_feed.html'], {
            'content': self,
            'load_code': self.code if self.code else "<script>%s</script>" % (TWITTER_LOAD_CODE)
        })


class AboutBlock(models.Model):

    title = models.CharField(max_length=200, blank=False)
    content = models.TextField(blank=False)
    image = MediaFileForeignKey(MediaFile, blank=True, null=True)
    image_position = models.CharField(max_length=200,
            choices=(('top', 'Top'), ('left', 'Left'), ('right', 'Right')))
    color = models.CharField(max_length=200, blank=False)
    offset_left = models.IntegerField(null=True, blank=True)
    offset_top = models.IntegerField(null=True, blank=True)

    class Meta:
        abstract = True
        verbose_name = _('about page block')
        verbose_name_plural = _('about page blocks')

    def render(self, **kwargs):
        return render_to_string(['content/about_block.html'], {'content': self})


class LoginForm(models.Model):
    """
    Login form
    """

    title = models.CharField(_('title'), max_length=200, blank=False)
    action_url = models.CharField(_('im url'), max_length=100, blank=False)
    display_forgot_password = models.BooleanField(default=False, null=False)
    next_url = models.CharField(max_length=255, null=True, blank=True)
    bottom_content = models.TextField(blank=True)
    logged_in_content = models.TextField(blank=True)

    class Meta:
        abstract = True
        verbose_name = _('login form')
        verbose_name_plural = _('login forms')

    def render(self, **kwargs):
        return render_to_string(['content/login_form.html'], {'content': self})


class IntroButton(models.Model):

    class Meta:
        abstract = True
        verbose_name = _('intro images')
        verbose_name_plural = _('intro images')

    image_1 = MediaFileForeignKey(MediaFile, blank=True, null=True,
            related_name="as_image1")
    image_2 = MediaFileForeignKey(MediaFile, blank=True, null=True,
            related_name="as_image2")
    image_3 = MediaFileForeignKey(MediaFile, blank=True, null=True,
            related_name="as_image3")
    link = models.CharField(max_length=255, blank=False, default="/welcome")
    link_title = models.CharField(max_length=255, blank=False, default="~okeanos")

    def render(self, **kwargs):
        return render_to_string(['content/intro_images.html'], {'content': self})



class TextileContent(models.Model):
    content = models.TextField()

    class Meta:
        abstract = True

    def render(self, **kwargs):
        return textile(self.content)

class IntroBlock(models.Model):
    title = models.CharField(_('title'), max_length=200, blank=False)
    content = models.TextField()
    class_name = models.CharField(_('class_name'), max_length=200, blank=True, null=True)
    image = MediaFileForeignKey(MediaFile, blank=True, null=True)

    class Meta:
        abstract = True

    def render(self, **kwargs):
        return render_to_string(['content/intro_block.html'], {'content': self})


class BlockColor(models.Model):
    title = models.CharField(_('title'), max_length=200, blank=False)
    content = models.TextField()
    color = models.CharField(max_length=200, blank=True, null=True)
    image = MediaFileForeignKey(MediaFile, blank=True, null=True)

    class Meta:
        abstract = True

    def render(self, **kwargs):
        return render_to_string(['content/block_with_color.html'], {'content': self})


class ClientDownload(models.Model):

    client = models.ForeignKey('cloudcms.Client')

    @property
    def media(self):
        return forms.Media(js=(
            settings.MEDIA_URL + 'cloudcms/' + 'js/' + 'client-downloads.js',))

    class Meta:
        abstract = True
        verbose_name = _('client download')
        verbose_name_plural = _('client downloads')

    def render(self, **kwrags):
        return render_to_string(['content/client_downloads.html'], {'content': self})


class StatsBlock(models.Model):

    @property
    def media(self):
        return forms.Media(js=(
            settings.MEDIA_URL + 'cloudcms/' + 'js/' + 'service-stats.js',))

    title = models.CharField(_('title'), max_length=200, blank=True)
    color = models.CharField(max_length=200, blank=True, null=True)
    running_vms = models.BooleanField(default=False, help_text = "ACTIVE VMs")
    spawned_vms = models.BooleanField(default=True, help_text= "All VMs")
    active_vms = models.BooleanField(default=True, help_text ="All VMs - \
        DELETED VMs")
    running_networks = models.BooleanField(default=False, help_text = "ACTIVE \
        networks")
    spawned_networks = models.BooleanField(default=True,help_text= "All \
        Networks")
    active_networks = models.BooleanField(default=False, help_text ="All \
        Networks - DELETED Networks")
    title_running_vms = models.CharField(max_length=255,blank=True, null=True)
    title_spawned_vms = models.CharField(max_length=255,blank=True, null=True)
    title_active_vms = models.CharField(max_length=255,blank=True, null=True)
    title_running_networks = models.CharField(max_length=255,blank=True, null=True)
    title_spawned_networks = models.CharField(max_length=255,blank=True, null=True)
    title_active_networks = models.CharField(max_length=255,blank=True, null=True)
    compute_url = models.CharField(max_length=255,blank=True, null=True)

    class Meta:
        abstract = True

    def render(self, **kwargs):
        return render_to_string(['content/stats_block.html'], {'content': self})

class IntroVideo(models.Model):

    @property
    def media(self):
        return forms.Media(js=(
            settings.MEDIA_URL + 'cloudcms/' + 'js/' + 'youtube.js',))

    image = MediaFileForeignKey(MediaFile, blank=True, null=True)
    youtube_id = models.CharField(max_length=255,blank=True, null=True)

    class Meta:
        abstract = True

    def render(self, **kwargs):
        return render_to_string(['content/intro_video.html'], {'content': self})
